library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;
use std.textio.all;

package mouseq_pkg is

  constant IWORDS : natural := 0; -- input registers
  constant OWORDS : natural := 31; -- output registers
  constant INTWORDS : natural := 7;    -- internal registers

  constant I2C_CLKS : unsigned(15 downto 0) := x"0100"; -- i2c clock prescale

  constant MEMWIDTH : natural := 16;               -- program mem word width, fixed
  constant WORDWIDTH : natural := 8;               -- reg width, fixed

  -- program memory size in words, address bus size
  constant MEMSIZE : natural := 256;
  constant MEMABITS : natural := integer(ceil(log2(real(MEMSIZE))));

  -- define a type for the program memory
  subtype RamCellType is std_logic_vector(MEMWIDTH-1 downto 0);
  type RamType is array(0 to MEMSIZE-1) of RamCellType;

  -- program memory initialization file
  constant PROG_FILENAME : string := "mouseq_mem.mif";

  -- register file size in words, address bus size
  constant REGSIZE : natural := IWORDS+OWORDS+INTWORDS;
  constant REGABITS : natural := integer(ceil(log2(real(REGSIZE))));

  -- define a type for the unified register space
  type RegType is array(0 to IWORDS+OWORDS+INTWORDS-1) of std_logic_vector(WORDWIDTH-1 downto 0);

  constant BUFFERIO : boolean := false; -- add register layer I/O or not
  constant BIG_ENDIAN : boolean := true; -- select endianness for multi-byte
  constant FAMILY : string := "NX"; -- select device family for mem implementation
  constant NX_PROGMEM : boolean := true; -- enable ECC prog memory implementation for nanoXplore
  constant SIMPLE_PROGMEM : boolean := true; -- select simple prog memory implementation (more resources with TMR)
  constant SIMPLE_REGFILE : boolean := false; -- select simple register file implementation (more resources)
  constant TMR : boolean := false; -- TMR constant used to set attributes
  constant ESSENTIALS_ONLY : boolean := false; -- disable NOP, MOV, ALU, LOAD, STOP and WAIT_MS

  constant ERR_SET_JUMP : boolean := true; -- enable watchdog
  constant ZERO_SET_JUMP : boolean := true; -- enable watchdog

  constant WATCHDOG_ON : boolean := false; -- enable watchdog
  constant WATCHDOG_VAL : natural := 1; -- watchdog value
  constant WATCHDOG_BITS : natural := integer(ceil(log2(real(WATCHDOG_VAL))));
  constant WATCHDOG_RESET_CYCLES : natural := 10; -- how many cycles to keep the watchdog reset high

  constant RESET_START_FROM : natural := 0; -- reset at a certain address to skip init

  constant NX_MEM_INIT : string; -- for memory instantiation in nanoXplore, defined below

  -- define a helper function to initialize the program memory
  impure function InitRamFromFile(RamFileName : in string) return RamType;
  impure function InitRamFirstCellFromFile(RamFileName : in string) return RamCellType;

  function vote(I1 : std_logic; I2 : std_logic; I3 : std_logic) return std_logic;
  function get_attr_radhard return string;

end package mouseq_pkg;

package body mouseq_pkg is

  impure function InitRamFromFile(RamFileName : in string) return RamType is
    file RamFile : text is in RamFileName;
    variable RamFileLine : line;
    variable temp_bv : bit_vector(MEMWIDTH-1 downto 0);
    variable RAM : RamType;
  begin
    for I in RamType'range loop
      readline(RamFile, RamFileLine);
      read(RamFileLine, temp_bv);
      RAM(I) := to_stdlogicvector(temp_bv);
    end loop;
    return RAM;
  end function;

  impure function InitRamFirstCellFromFile(RamFileName : in string) return RamCellType is
    file RamFile : text is in RamFileName;
    variable RamFileLine : line;
    variable temp_bv : bit_vector(MEMWIDTH-1 downto 0);
  begin
    readline(RamFile, RamFileLine);
    read(RamFileLine, temp_bv);
    return(to_stdlogicvector(temp_bv));
  end function;

  -- purpose: manually implement TMR to scrub memories
  function vote(I1 : std_logic; I2 : std_logic; I3 : std_logic)
    return std_logic is
  begin  -- function vote
    return ((I1 and I2) or (I2 and I3) or (I3 and I1));
  end function vote;

  -- purpose: set the syn_radhardlevel attribute if TMR is set
  function get_attr_radhard return string is
  begin
    if TMR then
      return "tmr";
    else
      return "none";
    end if;
  end function get_attr_radhard;

  constant NX_MEM_INIT : string := "001110100000000000," & "001100001100000000," &
    "001110011111000000," & "001110001100000000," &
    "001110100010000000," & "001101001100000000," &
    "001110100001000000," & "001110001100000000," &
    "001001001010110000," & "001110100101000000," &
    "001100100000000000," & "001111100101001000," &
    "001001000100100100," & "000101000110000001," &
    "001001000000000011," & "001000100011010000," &
    "001000000000000000," & "001001000010110100," &
    "001001000000000010," & "001000011110011000," &
    "000000000100000000," & "001001000000000010," &
    "001000011110011000," & "000000001000000000," &
    "001001000000000010," & "001000011110011000," &
    "000000010000000000," & "001001000000000010," &
    "001000011110011000," & "000000100000000000," &
    "001001000000000010," & "001000011110011000," &
    "000001000000000000," & "001001000000000010," &
    "001000011110011000," & "000010000000000000," &
    "001001000000000010," & "001000011110011000," &
    "000100000000000000," & "001001000000000010," &
    "001000011110011000," & "001000000000000000," &
    "001001000000000010," & "001110011110000000," &
    "000000000000000000," & "000010000000011111," &
    "001001100000000001," & "000010000000011111," &
    "001001101000000001," & "000010000000111111," &
    "001001100000000001," & "001110000000000100," &
    "000011011000000000," & "001001000100110100," &
    "000010000000111111," & "001001101000000001," &
    "001110000000000100," & "000011101100000000," &
    "001001000100110100," & "000010001111100001," &
    "001001000000000011," & "001110000000000100," &
    "000100000000000000," & "001001000010010000," &
    "000100000000000001," & "001001000000000001," &
    "000010010000100001," & "001001000000000011," &
    "001110000000000100," & "000100011100000000," &
    "001001000010010000," & "000100011100000001," &
    "001001000000000001," & "000010000000011111," &
    "001001100000000001," & "000010000000011111," &
    "001001101000000001," & "000010000001011111," &
    "001001100000000001," & "001110000000000100," &
    "000101001000000000," & "001001000101100100," &
    "000010000001011111," & "001001101000000001," &
    "001110000000000100," & "000101011100000000," &
    "001001000101100100," & "000010001111100001," &
    "001001000000000011," & "001110000000000100," &
    "000101110000000000," & "001001000010010000," &
    "000100000100000001," & "001001000000000001," &
    "000010010000100001," & "001001000000000011," &
    "001110000000000100," & "000110001100000000," &
    "001001000010010000," & "000100100000000001," &
    "001001000000000001," & "000010000000011111," &
    "001001100000000001," & "000010000000011111," &
    "001001101000000001," & "000010000010011111," &
    "001001100000000001," & "001110000000000100," &
    "000110111000000000," & "001001000110010100," &
    "000010000010011111," & "001001101000000001," &
    "001110000000000100," & "000111001100000000," &
    "001001000110010100," & "000010001111100001," &
    "001001000000000011," & "001110000000000100," &
    "000111100000000000," & "001001000010010000," &
    "000100001000000001," & "001001000000000001," &
    "000010010000100001," & "001001000000000011," &
    "001110000000000100," & "000111111100000000," &
    "001001000010010000," & "000100100100000001," &
    "001001000000000001," & "000010000000011111," &
    "001001100000000001," & "000010000000011111," &
    "001001101000000001," & "000010000100011111," &
    "001001100000000001," & "001110000000000100," &
    "001000101000000000," & "001001000111000100," &
    "000010000100011111," & "001001101000000001," &
    "001110000000000100," & "001000111100000000," &
    "001001000111000100," & "000010001111100001," &
    "001001000000000011," & "001110000000000100," &
    "001001010000000000," & "001001000010010000," &
    "000100001100000001," & "001001000000000001," &
    "000010010000100001," & "001001000000000011," &
    "001110000000000100," & "001001101100000000," &
    "001001000010010000," & "000100101000000001," &
    "001001000000000001," & "000010000000011111," &
    "001001100000000001," & "000010000000011111," &
    "001001101000000001," & "000010001000011111," &
    "001001100000000001," & "001110000000000100," &
    "001010011000000000," & "001001000111110100," &
    "000010001000011111," & "001001101000000001," &
    "001110000000000100," & "001010101100000000," &
    "001001000111110100," & "000010001111100001," &
    "001001000000000011," & "001110000000000100," &
    "001011000000000000," & "001001000010010000," &
    "000100010000000001," & "001001000000000001," &
    "000010010000100001," & "001001000000000011," &
    "001110000000000100," & "001011011100000000," &
    "001001000010010000," & "000100101100000001," &
    "001001000000000001," & "000010000000011111," &
    "001001100000000001," & "000010000000011111," &
    "001001101000000001," & "000010010000011111," &
    "001001100000000001," & "001110000000000100," &
    "001100001000000000," & "001001001000100100," &
    "000010010000011111," & "001001101000000001," &
    "001110000000000100," & "001100011100000000," &
    "001001001000100100," & "000010001111100001," &
    "001001000000000011," & "001110000000000100," &
    "001100110000000000," & "001001000010010000," &
    "000100010100000001," & "001001000000000001," &
    "000010010000100001," & "001001000000000011," &
    "001110000000000100," & "001101001100000000," &
    "001001000010010000," & "000100110000000001," &
    "001001000000000001," & "000010000000011111," &
    "001001100000000001," & "000010000000011111," &
    "001001101000000001," & "000010100000011111," &
    "001001100000000001," & "001110000000000100," &
    "001101111000000000," & "001001001001010100," &
    "000010100000011111," & "001001101000000001," &
    "001110000000000100," & "001110001100000000," &
    "001001001001010100," & "000010001111100001," &
    "001001000000000011," & "001110000000000100," &
    "001110100000000000," & "001001000010010000," &
    "000100011000000001," & "001001000000000001," &
    "000010010000100001," & "001001000000000011," &
    "001110000000000100," & "001110111100000000," &
    "001001000010010000," & "000100110100000001," &
    "001001000000000001," & "000100111000000001," &
    "001001001000000001," & "001110000000000100," &
    "001111011000000000," & "001001001010000100," &
    "001101000000000000," & "001010000011110010," &
    "001001001010110000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000," &
    "000000000000000000," & "000000000000000000";

end package body mouseq_pkg;
