-------------------------------------------------------------------------------
-- Title      : DIOT WIC Demonstrator
-- Project    : Distributed I/O Tier
-------------------------------------------------------------------------------
-- File       : diot_wic_top.vhd
-- Author     : Greg Daniluk <grzegorz.daniluk@cern.ch>
-- Company    : CERN (BE-CO-HT)
-- Standard   : VHDL
-------------------------------------------------------------------------------
-- Description:
-- DIOT top module with for Warm Interlocks Controller demo.
-------------------------------------------------------------------------------
--
-- Copyright (c) 2018 CERN
--
-- Copyright and related rights are licensed under the Solderpad Hardware
-- License, Version 0.51 (the “License”) (which enables you, at your option,
-- to treat this file as licensed under the Apache License 2.0); you may not
-- use this file except in compliance with the License. You may obtain a copy
-- of the License at http://solderpad.org/licenses/SHL-0.51.
-- Unless required by applicable law or agreed to in writing, software,
-- hardware and materials distributed under this License is distributed on an
-- “AS IS” BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
-- or implied. See the License for the specific language governing permissions
-- and limitations under the License.
--
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_misc.all;
use ieee.numeric_std.all;
use work.wic_pkg.all;
use work.diag_pkg.all;

library NX;
use NX.nxPackage.all;

entity diot_wic_top is
  port (
    clk_25m_i : in std_logic;
    button_i  : in std_logic;

    -- nanoFIP Wishbone
    dat_i  : in  std_logic_vector(7 downto 0);
    dat_o  : out std_logic_vector(7 downto 0);
    adr_o  : out std_logic_vector(9 downto 0);
    stb_o  : out std_logic;
    wclk_o : out std_logic;
    we_o   : out std_logic;
    cyc_o  : out std_logic;
    ack_i  : in  std_logic;
    rst_o  : out std_logic;
    -- nanoFIP control
    rstin_o    : out std_logic;
    nostat_o   : out std_logic;
    var3_acc_o : out std_logic;
    var3_rdy_i : in  std_logic;
    var2_acc_o : out std_logic;
    var2_rdy_i : in  std_logic;
    var1_acc_o : out std_logic;
    var1_rdy_i : in  std_logic;
    p3_lgth_o  : out std_logic_vector(2 downto 0);

    -- backplane connections
    -- in the first HW demonstrator not all slots are fully populated
    -- crate monitoring
    diag_scl_b  : inout std_logic;
    diag_sda_b  : inout std_logic;
    dbg : out std_logic_vector(4 downto 0);
    uart_tx : out std_logic;
    uart_rx : in  std_logic;
--
--    shared_bus : inout std_logic_vector(3 downto 0);
--    irq_n_i    : in  std_logic;

    leds_o  : out std_logic_vector(5 downto 0)
  );
end diot_wic_top;

architecture rtl of diot_wic_top is

  component DFN1C1
  port (
    d   : in  std_logic;
    clk : in  std_logic;
    clr : in  std_logic;
    q   : out std_logic);
  end component;

  constant c_IO_EMULATION : boolean := false;
  constant c_SLOT_NUM     : integer := 6;
  constant c_PERIOD       : unsigned(24 downto 0) := to_unsigned(25000000, 25);

  signal cnt       : unsigned(25 downto 0);
  signal leds      : std_logic_vector(5 downto 0);
  signal rst_cnt   : unsigned(15 downto 0) := (others=> '0');
  signal rst_n     : std_logic;
  signal scl_out, sda_out  : std_logic;
  signal scl_oen, sda_oen  : std_logic;

  signal slots_in        : t_wic_slot_in_array(c_SLOT_NUM-1 downto 0);
  signal slots_out       : t_wic_slot_out_array(c_SLOT_NUM-1 downto 0);
  signal diags_regs_out  : std_logic_vector(8*31-1 downto 0);
  signal diags           : t_diag_out;
  signal var1_rdy_synced : std_logic;
  signal var2_rdy_synced : std_logic;
  signal var3_rdy_synced : std_logic;

  signal s1_p_b  : std_logic_vector(15 downto 0);
  signal s1_n_b  : std_logic_vector(15 downto 0);
  signal s2_p_b  : std_logic_vector(21 downto 0); 
  signal s2_n_b  : std_logic_vector(21 downto 0); 
  signal s3_p_b  : std_logic_vector(13 downto 0);
  signal s3_n_b  : std_logic_vector(13 downto 0);
  signal s4_p_b  : std_logic_vector(13 downto 0);
  signal s4_n_b  : std_logic_vector(13 downto 0);
  signal s5_p_b  : std_logic_vector(13 downto 0);
  signal s5_n_b  : std_logic_vector(13 downto 0);
  -- slot6 not available in first HW demonstrator
  signal s7_p_b  : std_logic_vector(15 downto 0);
  signal s7_n_b  : std_logic_vector(15 downto 0);
  -- slot8 not available in first HW demonstrator

  signal trig   : std_logic;
  signal s0r0   : std_logic;
  signal s0r0_r : std_logic;

  signal button_i_n : std_logic;
  signal rst_n_i : std_logic;

  signal rst : std_logic;

  signal uart_transmit : std_logic;
  signal uart_tx_byte : std_logic_vector(7 downto 0);
  signal uart_received : std_logic;
  signal uart_rx_byte : std_logic_vector(7 downto 0);
  signal uart_is_receiving : std_logic;
  signal uart_is_transmitting : std_logic;
  signal uart_is_transmitting_r : std_logic;
  signal uart_recv_error : std_logic;

  signal uart_cnt : unsigned(5 downto 0) := (others => '0');
  signal uart_bytes : t_nf_var(0 to 33);

  signal clk_50m : std_logic;
  signal clk_50m_90 : std_logic;

  component uart is
    generic (
      CLOCK_DIVIDE : integer);          -- 1302
    port (
      clk : in std_logic;
      rst : in std_logic;
      rx  : in std_logic;
      tx  : out std_logic;
      transmit : in std_logic;
      tx_byte  : in std_logic_vector(7 downto 0);
      received : out std_logic;
      rx_byte  : out std_logic_vector(7 downto 0);
      is_receiving : out std_logic;
      is_transmitting : out std_logic;
      recv_error : out std_logic
      );
  end component uart;

begin

  wclk_o <= clk_50m;

  gc_glitch_filt_1: entity work.gc_glitch_filt
    generic map (
      g_len => 500)
    port map (
      clk_i   => clk_50m,
      rst_n_i => '1',
      dat_i   => button_i,
      dat_o   => rst_n_i);

  process(clk_50m, rst_n_i)
  begin
    if (rst_n_i = '0') then
      rst_cnt <= (others => '0');
    else
      if rising_edge(clk_50m) then
        if and_reduce(std_logic_vector(rst_cnt)) /= '1' then
          rst_cnt <= rst_cnt + 1;
        end if;
      end if;
    end if;
  end process;
  rst_n <= '1' when and_reduce(std_logic_vector(rst_cnt))='1' else
           '0';

-------------------------------------------------------------------------------
-- Synchronizers for RDY signals (40MHz clock domain in nanoFIP
-------------------------------------------------------------------------------
  cmp_sync_ffs_var1: entity work.gc_sync_ffs
    port map(
      clk_i    => clk_50m,
      rst_n_i  => rst_n,
      data_i   => var1_rdy_i,
      synced_o => var1_rdy_synced,
      npulse_o => open,
      ppulse_o => open);
  cmp_sync_ffs_var2: entity work.gc_sync_ffs
    port map(
      clk_i    => clk_50m,
      rst_n_i  => rst_n,
      data_i   => var2_rdy_i,
      synced_o => var2_rdy_synced,
      npulse_o => open,
      ppulse_o => open);
  cmp_sync_ffs_var3: entity work.gc_sync_ffs
    port map(
      clk_i    => clk_50m,
      rst_n_i  => rst_n,
      data_i   => var3_rdy_i,
      synced_o => var3_rdy_synced,
      npulse_o => open,
      ppulse_o => open);

-------------------------------------------------------------------------------

  cmp_wic_main: entity work.wic_main
  generic map (
    g_NUM => c_SLOT_NUM)
  port map (
    rst_n_i    => rst_n,
    clk_i      => clk_50m,
    slots_i    => slots_in,
    slots_o    => slots_out,

    diags_i    => diags,

    wb_cyc_o   => cyc_o,
    wb_stb_o   => stb_o,
    wb_we_o    => we_o,
    wb_adr_o   => adr_o,
    wb_dat_i   => dat_i,
    wb_dat_o   => dat_o,
    wb_ack_i   => ack_i,
    rst_o      => rst_o,

    rstin_o    => rstin_o,
    nostat_o   => nostat_o,
    var3_acc_o => var3_acc_o,
    var3_rdy_i => var3_rdy_synced,
    var2_acc_o => var2_acc_o,
    var2_rdy_i => var2_rdy_synced,
    var1_acc_o => var1_acc_o,
    var1_rdy_i => var1_rdy_synced,
    p3_lgth_o  => p3_lgth_o);

-------------------------------------------------------------------------------

  mouseq_nx_clkgen_1: entity work.mouseq_nx_clkgen
    port map (
      clk_i => clk_25m_i,
      clk_o => clk_50m,
      clk90_o => clk_50m_90,
      rst_o => open);
  
  mouseq_1: entity work.mouseq
    generic map (
      CLK_PERIOD => 20_000)
    port map (
      clk_i => clk_50m,
      clk90_i => clk_50m_90,
      rst_n_i => rst_n,
      ext_trig_i => trig,
      regs_in_i => (others => '0'),
      regs_out_o => diags_regs_out,
      scl_o => scl_out,
      scl_oen_o => scl_oen,
      scl_i => diag_scl_b,
      sda_o => sda_out,
      sda_oen_o => sda_oen,
      sda_i => diag_sda_b);

  process(clk_50m)
  begin
    if rising_edge(clk_50m) then
      if (rst_n = '0') then
        s0r0 <= '0';
        s0r0_r <= '0';
        trig <= '0';
      else
        s0r0 <= slots_out(0).relays(0);
        s0r0_r <= s0r0;
        if (s0r0 = '1' and s0r0_r = '0') then
          trig <= '1';
        else
          trig <= '0';
        end if;
      end if;
    end if;
  end process;

  diags.volt1(0)      <= diags_regs_out(01*8-1 downto 00*8);
  diags.volt1(1)      <= diags_regs_out(02*8-1 downto 01*8);
  diags.volt2(0)      <= diags_regs_out(03*8-1 downto 02*8);
  diags.volt2(1)      <= diags_regs_out(04*8-1 downto 03*8);
  diags.volt3(0)      <= diags_regs_out(05*8-1 downto 04*8);
  diags.volt3(1)      <= diags_regs_out(06*8-1 downto 05*8);
  diags.volt4(0)      <= diags_regs_out(07*8-1 downto 06*8);
  diags.volt4(1)      <= diags_regs_out(08*8-1 downto 07*8);
  diags.volt5(0)      <= diags_regs_out(09*8-1 downto 08*8);
  diags.volt5(1)      <= diags_regs_out(10*8-1 downto 09*8);
  diags.volt6(0)      <= diags_regs_out(11*8-1 downto 10*8);
  diags.volt6(1)      <= diags_regs_out(12*8-1 downto 11*8);
  diags.volt7(0)      <= diags_regs_out(13*8-1 downto 12*8);
  diags.volt7(1)      <= diags_regs_out(14*8-1 downto 13*8);
  diags.curr1(0)      <= diags_regs_out(15*8-1 downto 14*8);
  diags.curr1(1)      <= diags_regs_out(16*8-1 downto 15*8);
  diags.curr2(0)      <= diags_regs_out(17*8-1 downto 16*8);
  diags.curr2(1)      <= diags_regs_out(18*8-1 downto 17*8);
  diags.curr3(0)      <= diags_regs_out(19*8-1 downto 18*8);
  diags.curr3(1)      <= diags_regs_out(20*8-1 downto 19*8);
  diags.curr4(0)      <= diags_regs_out(21*8-1 downto 20*8);
  diags.curr4(1)      <= diags_regs_out(22*8-1 downto 21*8);
  diags.curr5(0)      <= diags_regs_out(23*8-1 downto 22*8);
  diags.curr5(1)      <= diags_regs_out(24*8-1 downto 23*8);
  diags.curr6(0)      <= diags_regs_out(25*8-1 downto 24*8);
  diags.curr6(1)      <= diags_regs_out(26*8-1 downto 25*8);
  diags.curr7(0)      <= diags_regs_out(27*8-1 downto 26*8);
  diags.curr7(1)      <= diags_regs_out(28*8-1 downto 27*8);
  diags.temp(0)       <= diags_regs_out(29*8-1 downto 28*8);
  diags.temp(1)       <= diags_regs_out(30*8-1 downto 29*8);
  diags.err           <= diags_regs_out(31*8-1 downto 30*8);

  diag_scl_b <= scl_out when scl_oen = '0' else 'Z';
  diag_sda_b <= sda_out when sda_oen = '0' else 'Z';
  dbg(0) <= uart_is_transmitting;
  dbg(1) <= uart_is_receiving;
  dbg(2) <= uart_recv_error;
  dbg(3) <= diag_scl_b;
  dbg(4) <= diag_sda_b;

  uart_inst : uart
    generic map (
      CLOCK_DIVIDE => 1302)              --651 for 25M, 1041 for 40M
    port map (
      clk             => clk_50m,
      rst             => rst,
      rx              => uart_rx,
      tx              => uart_tx,
      transmit        => uart_transmit,
      tx_byte         => uart_tx_byte,
      received        => uart_received,
      rx_byte         => uart_rx_byte,
      is_receiving    => uart_is_receiving,
      is_transmitting => uart_is_transmitting,
      recv_error      => uart_recv_error
      );

  uart_transmit <= not uart_is_transmitting;

  con_u_bytes: for i in 0 to 30 generate
    uart_bytes(i+2) <= diags_regs_out(8*(i + 1) - 1 downto 8*i);
  end generate con_u_bytes;
  uart_bytes(0) <= x"BE";
  uart_bytes(1) <= x"C0";
  uart_bytes(33) <= (others => '0');
  
  uart_cnt_proc: process(clk_50m)
  begin
    if rising_edge(clk_50m) then
      if rst_n = '0' then
        uart_is_transmitting_r <= '0';
        uart_tx_byte <= (others => '0');
        uart_cnt <= (others => '0');
      else
        uart_is_transmitting_r <= uart_is_transmitting;
        uart_tx_byte <= uart_bytes(to_integer(uart_cnt));
        if ((uart_is_transmitting_r = '0') and
            (uart_is_transmitting   = '1')) then
          if (uart_cnt = 33) then
            uart_cnt <= (others => '0');
          else
            uart_cnt <= uart_cnt + 1;
          end if;
        end if;
      end if;
    end if;
  end process;

  rst_inv_proc: process(clk_50m)
  begin
    if rising_edge(clk_50m) then
      rst <= not rst_n;
    end if;
  end process;
  
-------------------------------------------------------------------------------

GEN_IO_EMULATOR: if c_IO_EMULATION = true generate
  GEN_PORTS: for I in 0 to c_SLOT_NUM-1 generate
    cmp_wic_simulator: entity work.wic_simulator
    generic map (
      g_SIMULATION => false,
      g_ENABLE     => false)
    port map (
      rst_n_i => rst_n,
      clk_i   => clk_50m,
      slot_i  => slots_out(I),
      slot_o  => slots_in(I));
  end generate;
end generate;

-------------------------------------------------------------------------------
GEN_REAL_IO: if c_IO_EMULATION = false generate
  -- slots assignment
  slots_in(0).loops               <= s1_p_b(15 downto 0);
  slots_in(1).loops               <= s2_p_b(15 downto 0);
  slots_in(2).loops(15 downto 14) <= (others=>'0');
  slots_in(2).loops(13 downto 0)  <= s3_p_b(13 downto 0);
  slots_in(3).loops(15 downto 14) <= (others=>'0');
  slots_in(3).loops(13 downto 0)  <= s4_p_b(13 downto 0);
  slots_in(4).loops(15 downto 14) <= (others=>'0');
  slots_in(4).loops(13 downto 0)  <= s5_p_b(13 downto 0);
  slots_in(5).loops               <= s7_p_b(15 downto 0);

  s1_n_b(15 downto 0) <= slots_out(0).relays;
  s2_n_b(15 downto 0) <= slots_out(1).relays;
  s3_n_b(13 downto 0) <= slots_out(2).relays(13 downto 0);
  s4_n_b(13 downto 0) <= slots_out(3).relays(13 downto 0);
  s5_n_b(13 downto 0) <= slots_out(4).relays(13 downto 0);
  s7_n_b(15 downto 0) <= slots_out(5).relays;

  s1_p_b(15 downto 10) <= s1_n_b(15 downto 10);
  s2_p_b(15 downto 10) <= s2_n_b(15 downto 10);
  s3_p_b(13 downto 10) <= s3_n_b(13 downto 10);
  s4_p_b(13 downto 10) <= s4_n_b(13 downto 10);
  s5_p_b(13 downto 10) <= s5_n_b(13 downto 10);
  s7_p_b(15 downto 10) <= s7_n_b(15 downto 10);

  s1_p_b(9  downto 8) <= s1_n_b(8) & s1_n_b(9);
  s2_p_b(9  downto 8) <= s2_n_b(8) & s2_n_b(9);
  s3_p_b(9  downto 8) <= s3_n_b(8) & s3_n_b(9);
  s4_p_b(9  downto 8) <= s4_n_b(8) & s4_n_b(9);
  s5_p_b(9  downto 8) <= s5_n_b(8) & s5_n_b(9);
  s7_p_b(9  downto 8) <= s7_n_b(8) & s7_n_b(9);

  s1_p_b(7  downto 2) <= s1_n_b(7  downto 2);
  s2_p_b(7  downto 2) <= s2_n_b(7  downto 2);
  s3_p_b(7  downto 2) <= s3_n_b(7  downto 2);
  s4_p_b(7  downto 2) <= s4_n_b(7  downto 2);
  s5_p_b(7  downto 2) <= s5_n_b(7  downto 2);
  s7_p_b(7  downto 2) <= s7_n_b(7  downto 2);

  s1_p_b(1  downto 0) <= s1_n_b(0) & s1_n_b(1);
  s2_p_b(1  downto 0) <= s2_n_b(0) & s2_n_b(1);
  s3_p_b(1  downto 0) <= s3_n_b(0) & s3_n_b(1);
  s4_p_b(1  downto 0) <= s4_n_b(0) & s4_n_b(1);
  s5_p_b(1  downto 0) <= s5_n_b(0) & s5_n_b(1);
  s7_p_b(1  downto 0) <= s7_n_b(0) & s7_n_b(1);
end generate;

-------------------------------------------------------------------------------

  process(clk_50m)
  begin
    if rising_edge(clk_50m) then
      if rst_n = '0' then
        cnt  <= (others=>'0');
        leds <= "000001";
      else
        if cnt = to_unsigned(50000000, 26) then
          cnt <= (others=>'0');
          leds(5 downto 0) <= leds(4 downto 0) & leds(5);
        else
          cnt <= cnt + 1;
        end if;
      end if;
    end if;
  end process;

  GEN_LEDS_O: for I in 0 to 5 generate
    leds_o(I) <= '0' when leds(I) = '1' else
                 '1';
  end generate;

-------------------------------------------------------------------------------

end rtl;
