/*
 * Copyright (C) 2018 CERN
 * Author: Maciej Suminski <maciej.suminski@cern.ch>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CONTROLLER_H
#define CONTROLLER_H

#include <QtCore>
#include <QObject>
#include <QFrame>
#include <vector>
#include <memory>
#include <libmfipp.h>
#include <diot_crate.h>

class DiotIOController;
class QLabel;
class QPushButton;
class QStatusBar;

/**
 * Class representing the DIOT crate hardware in the Qt world.
 * It initalizes the hardware and provides signals and slots to other classes.
 */
class DiotHardware : public QObject {
    Q_OBJECT

public:
    /**
     * Constructor.
     * @param master_pci is the masterFIP index
     * @param address is the FIP agent address of the crate.
     * @param slots_nr is the number of I/O cards in the crate.
     */
    DiotHardware(int masterfip_index, int address, int slots_nr);

    /**
     * Start the WorldFIP cycle.
     */
    void Start();

    DIOT_Crate* GetCrate() const
    {
        return crate.get();
    }

public slots:
    /**
     * Change the output state for a card in a particular slot.
     * @param slot is the card number (indexing from 0).
     * @param state is the new output state.
     */
    void SetOutput(int slot, unsigned int state);

signals:
    /**
     * Signal emitted on every macrocycle, provides information about inputs state.
     * @param slot is the card number (indexing from 0).
     * @param state is the inputs state.
     */
    void InputChanged(int slot, unsigned int state);

    /**
     * Signal emitted whenever an input error occurs.
     * @param slot is the card number (indexing from 0).
     * @param state is the inputs state.
     */
    void InputErrChanged(int slot, unsigned int state);

    /**
     * Signal emitted on every macrocycle, provides information about fan state.
     * @param fan is the fan number (indexing from 0).
     * @param state is the fan state (true == operational, false == failure).
     * @param rpm is the fan speed (expressed in RPM).
     */
    void FanChanged(unsigned int fan, bool state, int rpm);

    /**
     * Signal emitted on every macrocycle, provides information about temperature.
     * @param sensor is the sensor number (indexing from 0).
     * @param state is the fan state (true == operational, false == failure).
     * @param temp is the temperature (Celsius degrees).
     */
    void TemperatureChanged(unsigned int sensor, bool state, int temp);

    /**
     * Signal emitted on every macrocycle, provides information about voltage.
     * @param volt is the voltage number (indexing from 0).
     * @param state is the voltage state (true == operational, false == failure).
     * @see DIOT_Crate::GetVoltages()
     */
    void VoltageChanged(unsigned int volt, bool state);

    /**
     * Signal emitted on every macrocycle, provides information about PSU state.
     * @param psu is the PSU number (indexing from 0).
     * @param voltage is the voltage.
     * @param current is the current.
     * @param temperature is the temperature.
     * @param power is the power.
     */
    void PSUChanged(unsigned int psu, float voltage, float current);

    /**
     * Signal emitted to update the data status.
     * @param status is the new data status.
     * @param stats contains statistics.
     * @see enum mstrfip_data_status
     */
    void StatusChanged(mstrfip_data_status status,
            const nanoFIP::DataStatusMap& stats);

private:
    std::unique_ptr<MasterFIP> mfip;
    std::unique_ptr<CycleSimple> cycle;
    std::unique_ptr<DIOT_Crate> crate;
    mstrfip_hw_cfg hw_cfg;
    mstrfip_sw_cfg sw_cfg;
};


/**
 * Widget representing a single I/O card in the crate.
 */
class DiotIOCard : public QFrame {
    Q_OBJECT

public:
    DiotIOCard(QWidget* parent, const DiotHardware& hw, int num, int idx);
    virtual ~DiotIOCard() {}

    unsigned int GetInput() const {
        return vec_to_int(inputs);
    }

    unsigned int GetOutput() const {
        return vec_to_int(outputs);
    }

public slots:
    void SetInput(unsigned int state);
    void SetInputErr(unsigned int state);
    void SetOutput(unsigned int state);

signals:
    void InputChanged(int slot, unsigned int state);
    void InputErrChanged(int slot, unsigned int state);
    void OutputChanged(int slot, unsigned int state);

protected:
    void testBtn_click(int slot, int io);
    void update_inputs();
    void update_outputs(int slot);

    static unsigned int vec_to_int(const std::vector<bool>& vec);
    static bool int_to_vec(unsigned int integer, std::vector<bool>& vec);

    std::vector<bool> inputs, outputs;
    std::vector<QLabel*> lbl_state;
    std::vector<QPushButton*> btn_clrerr;
    const int slot_nr;
    const int io_count;
};


/**
 * Widget grouping the I/O cards.
 */
class DiotIOController : public QFrame {
    Q_OBJECT

public:
    DiotIOController(QWidget* dialog, DiotHardware& hw, int slots_count);

public slots:
    void UpdateInput(int slot, unsigned int state);
    void UpdateInputErr(int slot, unsigned int state);

private:
    std::vector<std::unique_ptr<DiotIOCard>> io_slots;
    static constexpr unsigned int IO_PER_SLOT = 16;
};


/**
 * Widget displaying diagnostic data obtained from the crate.
 */
class DiotDiag : public QFrame {
    Q_OBJECT

public:
    DiotDiag(QWidget* parent, const DiotHardware& hw);
    virtual ~DiotDiag() {}

public slots:
    void UpdateFan(unsigned int fan, bool state, int rpm)
    {
        qDebug() << "fan " << fan << " state: " << state << " RPM: " << rpm;
        updateLabel(fan_lbl, fan, state, "Fan", rpm);
    }

    void UpdatePSU(unsigned int psu, float voltage, float current)
    {
        qDebug() << "PSU " << psu << " V: " << voltage << " I: " << current;
        updateLabel(psuv_lbl, psu, (voltage != 0xffff)?true:false, "Volts", voltage);
        updateLabel(psui_lbl, psu, (current != 0xffff)?true:false, "Amps", current);
    }

    void UpdateTemperature(unsigned int sensor, bool state, int temp)
    {
        qDebug() << "temperature " << sensor << " state: " << state << " value: " << temp;
        updateLabel(temp_lbl, sensor, state, "Temperature", temp);
    }

    void UpdateVoltage(unsigned int volt, bool state);

    void UpdateStatus(mstrfip_data_status status, const nanoFIP::DataStatusMap& stats);

private:
    // Helper function to update labels
    void updateLabel(std::vector<QLabel*> labels,
            unsigned int idx, bool state, const QString& desc, int val);
    void updateLabel(std::vector<QLabel*> labels,
            unsigned int idx, bool state, const QString& desc, float val);

    std::vector<QLabel*> fan_lbl;
    std::vector<QLabel*> temp_lbl;
    std::vector<QLabel*> volt_lbl;
    std::vector<QLabel*> psuv_lbl;
    std::vector<QLabel*> psui_lbl;
    std::vector<QLabel*> stat_lbl;
    QStatusBar* statusBar;
};

#endif /* CONTROLLER_H */
