/*
 * Copyright (C) 2018 CERN
 * Author: Maciej Suminski <maciej.suminski@cern.ch>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "diot_crate.h"
#include <iostream>
#include <iomanip>
#include <limits>
#include <cmath>

void DIOT_Crate::SetOutput(int slot, int idx, bool state)
{
    const int var_offset = getDataOffset(slot, idx);
    const uint8_t io_mask = getIOMask(idx);
    uint8_t data = GetProdVarData(var_offset);

    if ((data & io_mask) == state)
        return;     // no update needed

    if (state)
        data |= io_mask;
    else
        data &= ~io_mask;

    SetProdVarData(var_offset, data);
}


void DIOT_Crate::SetSlotOutputs(int slot, uint32_t state)
{
    const int var_offset = getDataOffset(slot, 0);

    {   // mutex locked scope
    auto lock = getProdLock();
    setProdDataLock(var_offset, (uint8_t)(state & 0xff), lock);
    setProdDataLock(var_offset+1, (uint8_t)(state >> 8 & 0xff), lock);
    setProdDataLock(var_offset+2, (uint8_t)(state >> 16 & 0xff), lock);
    setProdDataLock(var_offset+3, (uint8_t)(state >> 24 & 0xff), lock);
    }
}


float conv_direct_to_float(unsigned int lin_int)
{
    int mantissa = 0;
    int exponent = 0;

    if (lin_int & 0x400)
      mantissa = (int)(0xfffff800 | (lin_int & 0x7ff));
    else
      mantissa = (int)(lin_int & 0x3ff);

    if (lin_int & 0x00008000)
      exponent = (int)(0xffffffe0 | (lin_int & 0xf800) >> 11);
    else
      exponent = (int)((lin_int & 0xf800) >> 11);

    return(mantissa*std::pow(2.0, exponent));
}


float DIOT_Crate::GetPSUVoltage(unsigned int psu)
{
    if (psu >= 7)
        return std::numeric_limits<int>::min();

    int volts = 0;

    {   // mutex locked scope
    auto lock = getConsLock();
    volts = (int) getConsDataLock(PSUVOLT1 + psu * 2 + 1, lock) << 8;
    volts |= getConsDataLock(PSUVOLT1 + psu * 2, lock);
    }

    return volts;
}


float DIOT_Crate::GetPSUCurrent(unsigned int psu)
{
    if (psu >= 7)
        return std::numeric_limits<int>::min();

    int curr = 0;

    {   // mutex locked scope
    auto lock = getConsLock();
    curr = (int) getConsDataLock(PSUCURR1 + psu * 2 + 1, lock) << 8;
    curr |= getConsDataLock(PSUCURR1 + psu * 2, lock);
    }

    return curr;
}


int DIOT_Crate::GetTemp(unsigned int sensor)
{
    if (sensor >= TEMP_COUNT)
        return std::numeric_limits<int>::min();

    return GetConsVarData(TEMPERATURE0 + sensor);
}


uint32_t DIOT_Crate::getConsWord(int address)
{
    uint32_t ret = 0;

    {   // mutex locked scope
    auto lock = getConsLock();

    for (int i = 3; i >= 0; --i) {
        ret <<= 8;
        ret |= getConsDataLock(address + i, lock);
    }
    }

    return ret;
}

