/*
 * Copyright (C) 2018 CERN
 * Author: Maciej Suminski <maciej.suminski@cern.ch>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * libmfipp aka libmasterfiplusplus
 *
 * C++ wrapper for libmasterfip
 */

#ifndef LIBMFIPP_H
#define LIBMFIPP_H

#include <memory>
#include <vector>
#include <functional>
#include <unordered_map>

#include <masterfip/libmasterfip.h>

class CycleBase;
class AgentBase;

// Callback function signatures
typedef void (*VarMsgHandlerPtr)(mstrfip_dev*, mstrfip_data*, mstrfip_irq*);
typedef void (*IdentHandlerPtr)(mstrfip_dev*, mstrfip_irq*);
typedef std::function<void(mstrfip_dev*, mstrfip_data*, mstrfip_irq*)> VarMsgHandlerFn;
typedef std::function<void(mstrfip_dev*, mstrfip_irq*)> IdentHandlerFn;
typedef std::function<void(mstrfip_dev*, mstrfip_error_list)> ErrorHandlerFn;

///> Hash function for mstrip_error_list,
///> so it can be used with std::unordered_map
namespace std {
template<>
struct hash<mstrfip_error_list>
{
    size_t operator()(const mstrfip_error_list& a) const noexcept
    {
        // no need for any fancy algorithm, enum values are already unique
        return a;
    }
};
} // namespace std


/**
 * Represents a MasterFIP interface.
 *
 * It is created either using an existing mstrfip_dev pointer or via
 * OpenBy{LUN,FMC}() methods.
 */
class MasterFIP
{
public:
    MasterFIP(mstrfip_dev *device);
    ~MasterFIP();

    /**
     * Creates a MasterFIP instance by opening a specified LUN.
     */
    static std::unique_ptr<MasterFIP> OpenByLUN(uint32_t lun);

    /**
     * Creates a MasterFIP instance by opening a specified FMC.
     */
    static std::unique_ptr<MasterFIP> OpenByFMC(uint32_t fmc);

    /**
     * Searches for a MasterFip instance associated with a particular mstfip_dev.
     */
    static MasterFIP* FindByDev(mstrfip_dev *dev);

    /**
     * Configures the device.
     */
    void Configure(mstrfip_hw_cfg hw_cfg, mstrfip_sw_cfg sw_cfg);

    /**
     * Starts a macrocycle.
     */
    void StartCycle(CycleBase &cycle);

    /**
     * Stops a macrocycle.
     */
    void StopCycle();

    ///> MasterFIP interface states
    enum State { STOPPED, RUNNING };

    /**
     * Returns the current interface state.
     */
    State GetState() const
    {
        return state;
    }

    /**
     * Returns the device handle.
     */
    mstrfip_dev *GetDevice() const
    {
        return dev;
    }

    typedef std::unordered_map<mstrfip_error_list, int> MasterErrorMap;

    /**
     * Returns error counters.
     */
    const MasterErrorMap& GetErrors() const
    {
        return errors;
    }

    /**
     * Resets error counters.
     */
    void ClearErrors();

protected:
    ///> Error handler wrapper
    static void error_handler(mstrfip_dev *dev, mstrfip_error_list error);

    ///> Helper to get a MasterFIP error string preceded with a message
    static std::string errStr(const std::string& msg);

    ///> Device handle
    mstrfip_dev *dev;

    ///> FIP bus speed
    mstrfip_bitrate fbus_speed;

    ///> Flag indicating whether the device has been configured
    bool configured;

    ///> Device state
    State state;

    ///> Original error handler
    void (*orig_err_handler)(mstrfip_dev *dev, mstrfip_error_list error);

    ///> Error counter
    MasterErrorMap errors;

    ///> MasterFIP instances to resolve handlers
    static std::unordered_map<mstrfip_dev*, MasterFIP*> instances;
};


class WindowBase
{
public:
    ///> Window types
    enum Type { PERIODIC_VAR, APERIODIC_VAR, APERIODIC_MSG };

    WindowBase(CycleBase& cyc, Type win_type) : cycle(cyc), type(win_type) {}
    virtual ~WindowBase() = 0;

    Type GetType() const
    {
        return type;
    }

    /**
     * Returns number of variables/messages in the window.
     */
    size_t DataCount() const
    {
        return data.size();
    }

    /**
     * Returns true if there are no variables/messages in the window.
     */
    bool Empty() const
    {
        return data.empty();
    }

    virtual int Instantiate() = 0;

protected:
    ///> Parent macrocycle
    CycleBase& cycle;

    ///> Variables/messages
    std::vector<mstrfip_data*> data;

private:
    const Type type;
};


class WindowVarBase : public WindowBase
{
public:
    WindowVarBase(CycleBase& cycle, Type win_type) : WindowBase(cycle, win_type) {}
    virtual ~WindowVarBase() = 0;

    int AddVariable(int id, mstrfip_data_flags flags, int max_bsz,
            VarMsgHandlerPtr handler = nullptr)
    {
        return doAddVar(id, flags, max_bsz, handler) ? 0 : -1;
    }

    int AddVariable(int id, mstrfip_data_flags flags, int max_bsz,
            VarMsgHandlerFn handler);

protected:
    mstrfip_data* doAddVar(int id, mstrfip_data_flags flags, int max_bsz,
            VarMsgHandlerPtr handler = nullptr);

    static void handler_wrapper(mstrfip_dev *dev, mstrfip_data *var,
            mstrfip_irq *irq);

    ///> Map to resolve callback functions in handler_wrapper
    static std::unordered_map<mstrfip_data*, VarMsgHandlerFn> handler_map;
};


class WindowPeriodicVar : public WindowVarBase
{
public:
    WindowPeriodicVar(CycleBase& cycle) : WindowVarBase(cycle, PERIODIC_VAR) {}

    int Instantiate() override;
};


class WindowAperiodicVar : public WindowVarBase
{
public:
    WindowAperiodicVar(CycleBase& cycle)
        : WindowVarBase(cycle, APERIODIC_VAR),
        end_time(-1), diag(false), callback(nullptr) {}

    void SetEnd(int end_us)
    {
        end_time = end_us;
    }

    void EnableDiagnostics(bool enable)
    {
        diag = enable;
    }

    void SetCallback(IdentHandlerPtr cb)
    {
        callback = cb;
    }

    int AddIdent(uint32_t agent_addr);
    int Instantiate() override;

private:
    int end_time;
    bool diag;
    IdentHandlerPtr callback;
};


class WindowAperiodicMsg : public WindowBase
{
public:
    WindowAperiodicMsg(CycleBase& cycle, int end_us, int prod_size, int cons_size)
        : WindowBase(cycle, APERIODIC_MSG), end_time(end_us),
        prod_fifo_size(prod_size), cons_fifo_size(cons_size),
        cons_cb(nullptr), prod_cb(nullptr) {}

    int Instantiate() override;

    void SetConsMsgCb(VarMsgHandlerPtr cb)
    {
        cons_cb = cb;
    }

    void SetProdMsgCb(VarMsgHandlerPtr cb)
    {
        prod_cb = cb;
    }

private:
    int end_time, prod_fifo_size, cons_fifo_size;
    VarMsgHandlerPtr cons_cb, prod_cb;
};


class CycleBase
{
public:
    CycleBase(const MasterFIP &mfip, int length_us);
    virtual ~CycleBase();

    bool IsValid() const
    {
        return mstrfip_macrocycle_isvalid(dev, mcycle);
    }

    mstrfip_macrocycle *GetMacroCycle()
    {
        return mcycle;
    }

    virtual int Update() = 0;

    void AddAgent(AgentBase &agent) {
        agents.push_back(&agent);
    }

    int ReadRespTime();

protected:
    ///> Device handle
    mstrfip_dev *dev;

    ///> Macrocycle handle
    mstrfip_macrocycle *mcycle;

    ///> Cycle duration (microseconds)
    int length;

    ///> Agents registered for the cycle
    std::vector<AgentBase*> agents;
};


class CycleSimple : public CycleBase
{
public:
    CycleSimple(const MasterFIP &mfip, int length_us);

    WindowPeriodicVar& GetPeriodicVarWindow()
    {
        return win_per_var;
    }

    WindowAperiodicVar& GetAperiodicVarWindow()
    {
        return win_aper_var;
    }

protected:
    int Update() override;

    WindowPeriodicVar win_per_var;
    WindowAperiodicVar win_aper_var;
    bool instantiated;
};


///> Communication error types
enum AgentError {
    WRITE_FAILED,       ///< writing new value failed
    MISSED,             ///< missed data (not updated)
    NOT_FRESHED,        ///< other than 0x5 promptness & significance
    NOT_SIGNIFICANT,    ///< other than 0x4 promptness & significance
    TIMEOUT,            ///< time out, no reply from agent
    HW_ERR,             ///< hw bit error in the frame
    HW_BSZ,             ///< hw wrong number of bytes in the frame
    HW_CTRL,            ///< hw bad control byte in the frame
    HW_PDU,             ///< hw bad pdu byte in the frame
    UNKNOWN
};


namespace std {
template<>
struct hash<AgentError>
{
    size_t operator()(const AgentError& a) const noexcept
    {
        // no need for any fancy algorithm, enum values are already unique
        return a;
    }
};
} // namespace std


class AgentBase
{
public:
    AgentBase(unsigned int addr);
    virtual ~AgentBase() = 0;

    int GetAddress() const
    {
        return address;
    }

    typedef std::unordered_map<AgentError, unsigned int> AgentErrorMap;

    /**
     * Returns error counters.
     */
    const AgentErrorMap& GetErrors() const
    {
        return errors;
    }

    /**
     * Resets error counters.
     */
    void ClearErrors();

protected:
    /**
     * Reports an error occurence.
     * @param AgentError is the error type.
     */
    void reportError(AgentError error)
    {
        // TODO error handler callback?
        ++errors[error];
    }

    ///> Agent address on the FIP bus.
    const unsigned int address;

private:
    AgentErrorMap errors;
};


/**
 * Sleeps until a signal arrives
 *
 * Helper function for user applications. It will suspend the current thread
 * execution until a signal is received.
 */
int WaitForSignal();


#ifdef LIBMFIPP_DEBUG
    #define DBG(...) { fprintf(stderr, "%s:%d: ", __FILE__, __LINE__);\
                       fprintf(stderr, __VA_ARGS__); }
#else
    #define DBG(...)
#endif

#endif /* LIBMFIPP_H */
